{
PixaMSN
Copyright (C) 2005 Sergi Martinez (pixador@gmail.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
}

unit USw;

//{$DEFINE Debug}

interface

uses
  Windows, SysUtils, Classes, Graphics, Controls, Forms, //Dialogs,
  StdCtrls, ComCtrls, ScktComp, UUtils, ExtCtrls;

type
  TSbCallBack = procedure (aEmail: string) of object;
  TFlashWinCB = procedure (aHwnd: HWND) of object;

type
  TSwForm = class(TForm)
    Edit1: TEdit;
    RichEdit1: TRichEdit;
    StatusBar1: TStatusBar;
    SwSock: TClientSocket;
    Timer1: TTimer;
    procedure SwSockConnect(Sender: TObject; Socket: TCustomWinSocket);
    procedure SwSockRead(Sender: TObject; Socket: TCustomWinSocket);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure SwSockDisconnect(Sender: TObject; Socket: TCustomWinSocket);
    procedure Edit1KeyPress(Sender: TObject; var Key: Char);
    procedure Timer1Timer(Sender: TObject);
    procedure SwSockError(Sender: TObject; Socket: TCustomWinSocket;
      ErrorEvent: TErrorEvent; var ErrorCode: Integer);
  private
    FTrId   : cardinal;
    FCki    : string;
    FMe     : string;
    FYou    : string;
    FBuff   : string;
    FCloseCb: TSbCallBack;
    FNewSbCb: TSbCallBack;
    FFlashCb: TFlashWinCB;
    //FOwner  : TComponent;
    FCaller : boolean;
    FStarted: boolean;
    FReady  : boolean;
    FWasConn: boolean;
    FNeedSnd: boolean;

    procedure Processa(aLinia: string);
    procedure ProcessaMsg(aMsg: string);
    procedure Envia(aCmd, aParams: string; aAddCRLF: boolean = true);
    procedure EnviaMsg;
    procedure AddText(aText: string; aColor: TColor);
  public
    constructor Create(aOwner: TComponent; aMe, aYou: string; aCloseCb, aNewSbCb: TSbCallBack);
    destructor Destroy; override;
    procedure Start(aHost, aCki: string; aCaller: boolean);
    property Started: boolean read FStarted write FStarted;
    property WasConnected: boolean read FWasConn;
    property Ready: boolean read FReady;
    property FlashCB: TFlashWinCB write FFlashCB;
    { Public declarations }
  end;

var
  SwForm: TSwForm;

implementation

{$R *.DFM}

{ TSwForm }

constructor TSwForm.Create(aOwner: TComponent; aMe, aYou: string; aCloseCb, aNewSbCb: TSbCallBack);
begin
  inherited Create(aOwner);
  FFlashCB:=nil;
  FStarted:=false;
  FReady:=false;
  FWasConn:=false;
  FNeedSnd:=false;
  //FOwner:=aOwner;
  Caption:=aYou;
  FCloseCb:=aCloseCb;
  FNewSbCb:=aNewSbCb;
  FMe:=aMe;
  FYou:=aYou;
  FTrId:=1;
  Visible:=true;
end;

procedure TSwForm.Start(aHost, aCki: string; aCaller: boolean);
var
  i: integer;
begin
  if (SwSock.Active) then exit;
  FStarted:=true;
  FCki:=aCki;
  FCaller:=aCaller;
  i:=pos(':', aHost);
  SwSock.Address:='';
  SwSock.Service:='';
  SwSock.Host:=copy(aHost, 1, i-1);
  SwSock.Port:=strtoint(copy(aHost, i+1, length(aHost)));
  StatusBar1.Panels[1].Text:='Connexion en cours...';
  {$IFDEF Debug}log('PixaMSNlog.txt', 'Info: hte - '+SwSock.Host);{$ENDIF}
  {$IFDEF Debug}log('PixaMSNlog.txt', 'Info: port - '+inttostr(SwSock.Port));{$ENDIF}
  SwSock.Active:=true;
end;

destructor TSwForm.Destroy;
begin
  if (SwSock.Socket.Connected) then SwSock.Socket.SendText('OUT'+#13#10);
  inherited;
end;

procedure TSwForm.Envia(aCmd, aParams: string; aAddCRLF: boolean);
var
  s: string;
begin
  s:=Format('%s %d %s', [aCmd, FTrId, aParams]);
  inc(FTrId);
  if (aAddCRLF) then s:=s+#13#10;
  {$IFDEF Debug}log('PixaMSNlog.txt', 'SBS: '+s);{$ENDIF}
  SwSock.Socket.SendText(s);
end;

procedure TSwForm.AddText(aText: string; aColor: TColor);
begin
  with richedit1 do begin
    lines.beginupdate;
    SelLength:=-1;
    SelStart:=GetTextLen;
    SelAttributes.color:=aColor;
    Lines.Append(aText);
    lines.endupdate;
  end;
end;

procedure TSwForm.Processa(aLinia: string);
var
  tmp:string;
begin
  {$IFDEF Debug}log('PixaMSNlog.txt', 'SBR: '+aLinia);{$ENDIF}
  tmp:=copy(aLinia,1,3);

  if (tmp<'999') and (tmp>'000') then AddText('Erreur: '+tmp, clGreen);

  if (tmp='USR') then begin
    Envia('CAL', FYou);
  end;

  if (tmp='ANS') or (tmp='JOI') then begin
    FReady:=true;
    FWasConn:=true;
    StatusBar1.Panels[0].Text:='Ouvert';
    StatusBar1.Panels[1].Text:='';
    if ((FNeedSnd) and (Edit1.Text<>'')) then begin
      FNeedSnd:=false;
      EnviaMsg;
    end;
  end;

  if (tmp='NAK') then begin
    Timer1.Enabled:=false;
    StatusBar1.Panels[1].Text:='Erreur pendant l''envoi du message';
    SwSock.Socket.SendText('OUT'+#13#10);
  end;

  if (tmp='OUT') or (tmp='BYE') then begin
    SwSock.Socket.SendText('OUT'+#13#10);
  end;

end;

procedure TSwForm.ProcessaMsg(aMsg: string);
var
  i: integer;
  h: string;
begin
  {$IFDEF Debug}log('PixaMSNlog.txt', 'SBR: '+aMsg);{$ENDIF}
  i:=pos(#13#10#13#10, aMsg);
  if (i<0) then exit;
  h:=copy(aMsg, 1, i-1);
  delete(aMsg, 1, i+3);
  if (pos('text/plain', h)>0) then begin
    AddText(utf2asc(aMsg), clBlue);
    StatusBar1.Panels[1].Text:='';
    if assigned(FFlashCB) then FFlashCB(Handle);
  end else
  if (pos('text/x-msmsgscontrol', h)>0) then begin
    StatusBar1.Panels[1].Text:='User is typing...';
    Timer1.Enabled:=true;
  end;

end;

procedure TSwForm.SwSockConnect(Sender: TObject; Socket: TCustomWinSocket);
begin
  FBuff:='';
  if (FCaller) then Envia('USR', FMe+' '+FCki)
  else Envia('ANS', FMe+' '+FCki);
end;

procedure TSwForm.SwSockRead(Sender: TObject; Socket: TCustomWinSocket);
var
  i: integer;
  l: string;
  r: string;
begin
  repeat
    r:=socket.ReceiveText;
    if (r='') then exit;
    FBuff:=FBuff+r;
    i:=pos(#10, FBuff);
    while i>0 do begin
      l:=copy(FBuff,1,i-2);
      if (copy(l, 1, 3)<>'MSG') then begin
        FBuff:=copy(FBuff,i+1,22222);
        Processa(l);
      end
      else begin
        i:=strtoint(param(l, 4));
        i:=i+length(l)+2;
        if (length(FBuff)<i) then break;
        l:=copy(FBuff, 1, i);
        delete(FBuff, 1, i);
        ProcessaMsg(l);
      end;
      i:=pos(#10, FBuff);
    end;
  until (r='');
end;

procedure TSwForm.SwSockDisconnect(Sender: TObject; Socket: TCustomWinSocket);
begin
  //FStarted:=false;
  FReady:=false;
  StatusBar1.Panels[0].Text:='Ferm';
end;

procedure TSwForm.SwSockError(Sender: TObject; Socket: TCustomWinSocket;
  ErrorEvent: TErrorEvent; var ErrorCode: Integer);
begin
  ErrorCode:=0;
  //FStarted:=false;
  FReady:=false;
  StatusBar1.Panels[0].Text:='Erreur';
end;

procedure TSwForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if (SwSock.Socket.Connected) then SwSock.Socket.SendText('OUT'+#13#10);
  FCloseCb(FYou);
end;

procedure TSwForm.EnviaMsg;
var
  msg: string;
begin
  msg:=    'MIME-Version: 1.0'+#13#10;
  msg:=msg+'Content-Type: text/plain; charset=UTF-8'+#13#10;
  msg:=msg+'X-MMS-IM-Format: FN=Arial; EF=; CO=000000; CS=0; PF=31'+#13#10#13#10;
  msg:=msg+asc2utf(Edit1.Text);
  Envia('MSG', 'N '+inttostr(length(msg))+#13#10+msg, false);
  AddText(Edit1.Text, clRed);
  Edit1.Clear;
end;

procedure TSwForm.Edit1KeyPress(Sender: TObject; var Key: Char);
begin
  if (key<>#13) then exit;
  if (FReady) then EnviaMsg
  else begin
    FNeedSnd:=true;
    if ((FWasConn) and (FStarted)) then begin
      FStarted:=false;
      StatusBar1.Panels[1].Text:='Connexion en cours...';
      FNewSbCb(FYou);
    end;
  end;
  key:=#0;
end;

procedure TSwForm.Timer1Timer(Sender: TObject);
begin
  Timer1.Enabled:=false;
  StatusBar1.Panels[1].Text:='';
end;

end.
